import difflib
import os


def compare_files(file1, file2):
    with open(file1, 'r') as f1, open(file2, 'r') as f2:
        diff = difflib.unified_diff(f1.readlines(), f2.readlines())
        return ''.join(diff)


def generate_procedure(file1, file2):
    diff = compare_files(file1, file2)
    lines = diff.split('\n')
    insert_lines = []
    delete_lines = []
    insert_query = []
    delete_query = []
    for line in lines:
        if line.startswith('+'):
            insert_lines.append(line[1:])
        elif line.startswith('-'):
            delete_lines.append(line[1:])
    for insert_line in insert_lines[1:]:
        values = insert_line.split('|')
        values2=values[1:]
        for i in range(len(values2)):
            if isinstance(values2[i], str):
                values2[i] = "'{}'".format(values2[i])
        insert_query.append('INSERT INTO {} VALUES ({})'.format(values[0],', '.join(values2)))

    # 删除先不写了，用不上
    procedure = '''\
CREATE OR REPLACE FUNCTION restore_backup(numbegin NUMERIC,numend NUMERIC)
RETURNS NUMERIC AS $$
DECLARE a int DEFAULT 1;
DECLARE
BEGIN
    a:=numbegin;
    while a < numend LOOP
        {};
        a = a + 1 ;
    END LOOP;
END;
$$ LANGUAGE plpgsql;
SELECT restore_backup(0,2);
'''.format(';\n        '.join(insert_query))
    with open('restore_backup.sql', 'w') as f:
        f.write(procedure)
    return procedure


def run_compare():
    backup_files = [f for f in os.listdir('.') if f.startswith('backup_')]
    if len(backup_files) < 2:
        print('\033[33m·\033[0m数据库备份数量不够2个，请先gensp backup备份数据库')
    file1 = backup_files[0]
    file2 = backup_files[1]
    # 比较时间戳
    if int(file1.split('_')[1].split('.')[0]) > int(file2.split('_')[1].split('.')[0]):
        file1, file2 = file2, file1
    procedure = generate_procedure(file1, file2)
    print(procedure)


import sys
from .login import login
from .backup import get_backup
from .compare import run_compare
import os


def main():
    if len(sys.argv) == 2:
        if sys.argv[1] == 'login':
            login()
        elif sys.argv[1] == 'backup':
            file_number = 0
            for file in os.listdir():
                if file.startswith('backup_') and file.endswith('.sql'):
                    file_number += 1
            print('\033[32m·\033[0m已存在备份数量: ' + str(file_number))
            if file_number >= 2:
                print('\033[33m·\033[0m备份数量已满，请执行gsp compare生成存储过程，或执行gsp clear清空备份')
                sys.exit()
            database_name = input('\033[34m·\033[0mdatabase: ').strip()
            print('\033[32m·\033[0m备份中……')
            get_backup(database_name)
        elif sys.argv[1] == 'compare':
            print('\033[32m·\033[0m生成存储过程如下: ')
            run_compare()
        elif sys.argv[1] == 'clear':
            for file in os.listdir():
                if file.startswith('backup_') and file.endswith('.sql'):
                    os.remove(file)
            print('\033[32m·\033[0mcompleted!')
        elif sys.argv[1] == 'help':
            print('\033[32m·\033[0m参数介绍：')
            print('\033[32m·\033[0mlogin：注册数据库基本信息')
            print('\033[32m·\033[0mbackup：备份数据库')
            print('\033[32m·\033[0mcompare：比较备份并生成存储过程')
            print('\033[32m·\033[0mclear：清除本地数据库备份')
            print('\033[32m·\033[0mhelp：帮助')
        elif sys.argv[1] == 'lasttime':
            with open('restore_backup.sql', 'r') as f:
                content = f.read()
                print(content)



        else:
            print("\033[31m·\033Invalid argument")
    else:
        print("\033[31m·\033Invalid number of arguments")


if __name__ == '__main__':
    main()
